from __future__ import annotations

from typing import Any


class LinWheelApiError(Exception):
    def __init__(
        self,
        message: str,
        status_code: int,
        response_body: Any = None,
    ) -> None:
        super().__init__(message)
        self.status_code = status_code
        self.response_body = response_body


class LinWheelAuthError(LinWheelApiError):
    def __init__(self, message: str, response_body: Any = None) -> None:
        super().__init__(message, 401, response_body)
